/// <reference types="react" />
import { CSSProperties } from "react";
import { CSSTransitionProps, CSSTransitionDelay } from "./csstransition";
export interface TransitionState {
    id?: StateID;
    style?: CSSProperties;
    className?: string;
    inTransition?: boolean;
}
export declare enum StateID {
    EntryPoint = 0,
    DefaultInit = 1,
    ActiveInit = 2,
    AppearInit = 3,
    Default = 4,
    Active = 5,
    AppearPending = 6,
    AppearTriggered = 7,
    AppearStarted = 8,
    EnterPending = 9,
    EnterTriggered = 10,
    EnterStarted = 11,
    LeavePending = 12,
    LeaveTriggered = 13,
    LeaveStarted = 14,
}
export declare const StateIDList: StateID[];
export declare enum ActionID {
    Init = 0,
    Mount = 1,
    TransitionInit = 2,
    TransitionTrigger = 3,
    TransitionStart = 4,
    TransitionComplete = 5,
}
export declare type ActionPropKeys = "active" | "transitionAppear" | "transitionDelay" | "defaultStyle" | "activeStyle" | "appearStyle" | "enterStyle" | "leaveStyle" | "appearInitStyle" | "enterInitStyle" | "leaveInitStyle" | "defaultClassName" | "activeClassName" | "appearClassName" | "enterClassName" | "leaveClassName" | "appearInitClassName" | "enterInitClassName" | "leaveInitClassName";
export declare const actionPropKeys: ActionPropKeys[];
export declare type ActionProps = {
    [P in ActionPropKeys]?: CSSTransitionProps[P];
};
export declare type Action = {
    kind: ActionID;
    props: ActionProps;
};
export declare const transitionNames: string[];
export declare function hasTransition(name: string, props: any): boolean;
export declare function getDelay(name: string, delay: CSSTransitionDelay): number;
export declare function getState(id: StateID, name: string, props: any, params?: {
    init?: boolean;
}): TransitionState;
export declare function stateFunc(id: StateID, name: string, params?: {
    init?: boolean;
}): (props: ActionProps) => TransitionState;
export declare const activeInitState: (props: ActionProps) => TransitionState;
export declare const defaultInitState: (props: ActionProps) => TransitionState;
export declare const appearInitState: (props: ActionProps) => TransitionState;
export declare const activeState: (props: ActionProps) => TransitionState;
export declare const defaultState: (props: ActionProps) => TransitionState;
export declare const appearPendingState: (props: ActionProps) => TransitionState;
export declare const enterPendingState: (props: ActionProps) => TransitionState;
export declare const leavePendingState: (props: ActionProps) => TransitionState;
export declare const appearTriggeredState: (props: ActionProps) => TransitionState;
export declare const enterTriggeredState: (props: ActionProps) => TransitionState;
export declare const leaveTriggeredState: (props: ActionProps) => TransitionState;
export declare const appearStartedState: (props: ActionProps) => TransitionState;
export declare const enterStartedState: (props: ActionProps) => TransitionState;
export declare const leaveStartedState: (props: ActionProps) => TransitionState;
export declare type Reducer = (stateID: StateID, action: Action) => {
    state: TransitionState;
    pending?: ActionID;
    completed?: boolean;
};
export declare const reducer: Reducer;
