/* tslint:disable: variable-name no-switch-case-fall-through */
import { resolveTransit } from "./transit";
export var StateID;
(function (StateID) {
    StateID[StateID["EntryPoint"] = 0] = "EntryPoint";
    StateID[StateID["DefaultInit"] = 1] = "DefaultInit";
    StateID[StateID["ActiveInit"] = 2] = "ActiveInit";
    StateID[StateID["AppearInit"] = 3] = "AppearInit";
    StateID[StateID["Default"] = 4] = "Default";
    StateID[StateID["Active"] = 5] = "Active";
    StateID[StateID["AppearPending"] = 6] = "AppearPending";
    StateID[StateID["AppearTriggered"] = 7] = "AppearTriggered";
    StateID[StateID["AppearStarted"] = 8] = "AppearStarted";
    StateID[StateID["EnterPending"] = 9] = "EnterPending";
    StateID[StateID["EnterTriggered"] = 10] = "EnterTriggered";
    StateID[StateID["EnterStarted"] = 11] = "EnterStarted";
    StateID[StateID["LeavePending"] = 12] = "LeavePending";
    StateID[StateID["LeaveTriggered"] = 13] = "LeaveTriggered";
    StateID[StateID["LeaveStarted"] = 14] = "LeaveStarted";
})(StateID || (StateID = {}));
export var StateIDList = [
    StateID.ActiveInit, StateID.DefaultInit, StateID.AppearInit,
    StateID.Active, StateID.Default,
    StateID.AppearPending, StateID.AppearTriggered, StateID.AppearStarted,
    StateID.EnterPending, StateID.EnterTriggered, StateID.EnterStarted,
    StateID.LeavePending, StateID.LeaveTriggered, StateID.LeaveStarted,
];
export var ActionID;
(function (ActionID) {
    ActionID[ActionID["Init"] = 0] = "Init";
    ActionID[ActionID["Mount"] = 1] = "Mount";
    ActionID[ActionID["TransitionInit"] = 2] = "TransitionInit";
    ActionID[ActionID["TransitionTrigger"] = 3] = "TransitionTrigger";
    ActionID[ActionID["TransitionStart"] = 4] = "TransitionStart";
    ActionID[ActionID["TransitionComplete"] = 5] = "TransitionComplete";
})(ActionID || (ActionID = {}));
export var actionPropKeys = [
    "active",
    "transitionAppear",
    "transitionDelay",
    "defaultStyle",
    "activeStyle",
    "appearStyle",
    "enterStyle",
    "leaveStyle",
    "appearInitStyle",
    "enterInitStyle",
    "leaveInitStyle",
    "defaultClassName",
    "activeClassName",
    "appearClassName",
    "enterClassName",
    "leaveClassName",
    "appearInitClassName",
    "enterInitClassName",
    "leaveInitClassName",
];
export var transitionNames = ["enter", "leave", "appear"];
export function hasTransition(name, props) {
    var result = props[name + "Style"] !== undefined || props[name + "ClassName"] !== undefined;
    return !result && name === "appear"
        ? hasTransition("enter", props)
        : result;
}
export function getDelay(name, delay) {
    if (!delay) {
        return 0;
    }
    if (typeof delay === "number") {
        return delay;
    }
    return delay[name] ? delay[name] : 0;
}
export function getState(id, name, props, params) {
    if (params === void 0) { params = {}; }
    if (name === "appear" && !props.appearStyle && !props.appearClassName) {
        return getState(id, "enter", props, params);
    }
    var style;
    var className;
    var inTransition = false;
    if (params.init) {
        style = props[name + "InitStyle"];
        className = props[name + "InitClassName"];
        if (style === undefined && className === undefined) {
            if (name === "enter" || name === "appear") {
                style = props.defaultStyle;
                className = props.defaultClassName;
            }
            else if (name === "leave") {
                style = props.activeStyle;
                className = props.activeClassName;
            }
        }
    }
    else {
        style = props[name + "Style"];
        className = props[name + "ClassName"];
        if (["enter", "appear", "leave"].indexOf(name) >= 0) {
            inTransition = true;
            style = resolveTransit(style, getDelay(name, props.transitionDelay));
        }
    }
    style = style || {};
    className = className || "";
    return {
        id: id,
        style: style,
        className: className,
        inTransition: inTransition,
    };
}
export function stateFunc(id, name, params) {
    if (params === void 0) { params = {}; }
    return function (props) { return getState(id, name, props, params); };
}
export var activeInitState = stateFunc(StateID.ActiveInit, "active");
export var defaultInitState = stateFunc(StateID.DefaultInit, "default");
export var appearInitState = stateFunc(StateID.AppearInit, "appear", { init: true });
export var activeState = stateFunc(StateID.Active, "active");
export var defaultState = stateFunc(StateID.Default, "default");
export var appearPendingState = stateFunc(StateID.AppearPending, "appear", { init: true });
export var enterPendingState = stateFunc(StateID.EnterPending, "enter", { init: true });
export var leavePendingState = stateFunc(StateID.LeavePending, "leave", { init: true });
export var appearTriggeredState = stateFunc(StateID.AppearTriggered, "appear");
export var enterTriggeredState = stateFunc(StateID.EnterTriggered, "enter");
export var leaveTriggeredState = stateFunc(StateID.LeaveTriggered, "leave");
export var appearStartedState = stateFunc(StateID.AppearStarted, "appear");
export var enterStartedState = stateFunc(StateID.EnterStarted, "enter");
export var leaveStartedState = stateFunc(StateID.LeaveStarted, "leave");
export var reducer = function (stateID, action) {
    var props = action.props;
    switch (action.kind) {
        case ActionID.Init:
            if (stateID !== StateID.EntryPoint) {
                throw new Error("invalid entrypoint");
            }
            if (props.active) {
                if (props.transitionAppear) {
                    return { state: appearInitState(props) };
                }
                return { state: activeInitState(props) };
            }
            if (!props.transitionAppear && props.active) {
                return { state: activeInitState(props) };
            }
            return { state: defaultInitState(props) };
        case ActionID.Mount:
            switch (stateID) {
                case StateID.AppearInit:
                    return reducer(stateID, { kind: ActionID.TransitionTrigger, props: props });
                default:
                    return null;
            }
        case ActionID.TransitionInit:
            var nextState = void 0;
            switch (stateID) {
                case StateID.DefaultInit:
                case StateID.Default:
                    if (!hasTransition("enter", props)) {
                        return { state: activeState(props), completed: true };
                    }
                    nextState = enterPendingState(props);
                    break;
                case StateID.ActiveInit:
                case StateID.Active:
                    if (!hasTransition("leave", props)) {
                        return { state: defaultState(props), completed: true };
                    }
                    nextState = leavePendingState(props);
                    break;
                case StateID.AppearInit:
                    if (!hasTransition("appear", props)) {
                        return { state: activeState(props), completed: true };
                    }
                    nextState = appearPendingState(props);
                    break;
                default:
                    throw new Error("invalid state transition from " + StateID[stateID]);
            }
            ;
            return { state: nextState, pending: ActionID.TransitionTrigger };
        case ActionID.TransitionStart:
            switch (stateID) {
                case StateID.EnterTriggered:
                    return { state: enterStartedState(props) };
                case StateID.LeaveTriggered:
                    return { state: leaveStartedState(props) };
                case StateID.AppearTriggered:
                    return { state: appearStartedState(props) };
                default:
                    // We don't error out, because the workaround for transitionstart
                    // could happen after transitionend.
                    return null;
            }
        case ActionID.TransitionComplete:
            switch (stateID) {
                case StateID.AppearStarted:
                case StateID.AppearTriggered:
                case StateID.EnterTriggered:
                case StateID.EnterStarted:
                    return { state: activeState(props), completed: true };
                case StateID.LeaveTriggered:
                case StateID.LeaveStarted:
                    return { state: defaultState(props), completed: true };
                default:
                    throw new Error("invalid state transition from " + StateID[stateID]);
            }
        case ActionID.TransitionTrigger:
            switch (stateID) {
                case StateID.ActiveInit:
                case StateID.Active:
                case StateID.DefaultInit:
                case StateID.Default:
                case StateID.AppearInit:
                    return reducer(stateID, { kind: ActionID.TransitionInit, props: props });
                case StateID.EnterPending:
                    if (props.active) {
                        return { state: enterTriggeredState(props) };
                    }
                    return { state: defaultState(props), completed: true };
                case StateID.LeavePending:
                    if (!props.active) {
                        return { state: leaveTriggeredState(props) };
                    }
                    return { state: activeState(props), completed: true };
                case StateID.AppearPending:
                    if (props.active) {
                        return { state: appearTriggeredState(props) };
                    }
                    return { state: defaultState(props), completed: true };
                case StateID.EnterTriggered:
                    return { state: defaultState(props), completed: true };
                case StateID.LeaveTriggered:
                    return { state: activeState(props), completed: true };
                case StateID.AppearTriggered:
                    return { state: defaultState(props), completed: true };
                case StateID.AppearStarted:
                case StateID.EnterStarted:
                    return { state: leaveStartedState(props) };
                case StateID.LeaveStarted:
                    return { state: enterStartedState(props) };
                default:
                    throw new Error("invalid state transition from " + StateID[stateID]);
            }
        default:
    }
    throw new Error("unexpected error");
};

//# sourceMappingURL=reducer.js.map
